#! /usr/bin/env python

## Copyright 2000-2001 by LivingLogic AG, Bayreuth, Germany.
## Copyright 2000-2001 by Walter Drwald
##
## All Rights Reserved
##
## Permission to use, copy, modify, and distribute this software and its documentation
## for any purpose and without fee is hereby granted, provided that the above copyright
## notice appears in all copies and that both that copyright notice and this permission
## notice appear in supporting documentation, and that the name of LivingLogic AG or
## the author not be used in advertising or publicity pertaining to distribution of the
## software without specific, written prior permission.
##
## LIVINGLOGIC AG AND THE AUTHOR DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
## INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
## LIVINGLOGIC AG OR THE AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
## DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
## IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
## IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

"""
This module adds ANSI color capability to an output stream.

This is done by wrapping an output stream in a Stream object:

	import ansistyle
	import sys
	s = ansistyle.Stream(sys.stdout)

In addition to the write method that writes to the underlying
stream, ansistyle.Stream has three methods for handling color.
pushColor() takes an integer between 0 and 15. These colors
are equivalent to the following escape sequences:

	color 0: "\033[30m"
	...
	color 7: "\033[37m"
	color 8: "\033[1;30m"
	...
	color 15: "\033[1;37m"

Now you can write to the stream with write(),
and the output will appear in that color. Returning
to the previously active color is possible with popColor():

	s.write("spam")
	s.pushColor(13)
	s.write("eggs") # will be in color 13
	s.popColor() # returns the currently active color and returns to the previous one ;)

As the name suggests, calls to pushColor() and popColor()
can be "nested".

Note that the ANSI escape sequences won't be written
to the stream immediately when calling pushColor(),
but only when output is written to the stream. This
means that the final call to popColor() in the above
example will not switch back to the default color.
To achieve this, call the method finish() at the end
of all your write statement.

	s.pushColor(7)
	s.write("spam")
	s.pushColor(13)
	s.write("eggs")
	s.popColor()
	s.popColor()
	s.finish()

There is a second level API for ansistyle.Stream. The class
ansistyle.Text is a "colored text sequence" and can
be written to an ansistyle.Stream as well.

The color of a Text object is available via the method
getColor(). Text.getColor() returns the value of the
class attribute color, so derived classes can either
use their own color attribute, or completely overwrite
getColor(). Ansistyle contains classes for all the 16 colors
named Text0, ..., Text15.

In addition to the value 0-15, -1 is allowed as a return
value for getColor(). This means "do not change the current
color".

A Text object is a sequence that may consist of strings and
Text objects. So the above example can be written as:

	s.write(Text7("spam", Text13("eggs")))
	s.finish()

Text objects have a __str__ method that can be used directly
to print the object (the call to finish() is included).
So the above example can be written as follows:

	s.write(str(Text7("spam", Text13("eggs"))))

"""

__version__ = tuple(map(int, "$Revision: 1.10 $"[11:-2].split(".")))
# $Source: /data/cvsroot/LivingLogic/ansistyle/ansistyle.py,v $

from _ansistyle import Stream

class StringBuffer:
	def __init__(self):
		self.buffer = []
	def write(self, text):
		self.buffer.append(text)

class Text:
	"""
	a colored string, colors can be numbers from
	0 to 15 for the 16 ANSI colors or -1 if this
	Text should use the current color. A Text object
	is a sequence, the sequence items may either be
	strings or Text objects themselves.
	"""
	color = -1 # the default color, which means "do not switch color"

	def __init__(self, *content):
		self.content = list(content)

	def __getitem__(self, index):
		return self.content[index]

	def __setitem__(self, index, value):
		self.content[index] = value

	def __delitem__(self, index):
		del self.content[index]

	def __getslice__(self, index1, index2):
		return self.content[index1:index2]

	def __setslice__(self, index1, index2, sequence):
		self.content[index1:index2] = sequence

	def __delslice__(self, index1, index2):
		del self.content[index1:index2]

	def __len__(self):
		return len(self.content)

	def append(self, *others):
		self.content.extend(others)

	def insert(self, index, *others):
		self.content[index:index] = others

	def getColor(self):
		"""
		return the color of this string. Overwrite this method
		or the class attribute color to change the behaviour.
		"""
		return self.color

	def __repr__(self):
		return self.__class__.__name__ + "(" + ", ".join([ repr(text) for text in self.content ]) + ")"

	def __str__(self):
		"""
		return the resulting string with ANSI color escape sequences.
		"""
		stream = Stream(StringBuffer())
		stream.write(self)
		stream.finish() # return to the default color
		return "".join(stream.stream.buffer)

class Text0(Text):
	color = 0

class Text1(Text):
	color = 1

class Text2(Text):
	color = 2

class Text3(Text):
	color = 3

class Text4(Text):
	color = 4

class Text5(Text):
	color = 5

class Text6(Text):
	color = 6

class Text7(Text):
	color = 7

class Text8(Text):
	color = 8

class Text9(Text):
	color = 9

class Text10(Text):
	color = 10

class Text11(Text):
	color = 11

class Text12(Text):
	color = 12

class Text13(Text):
	color = 13

class Text14(Text):
	color = 14

class Text15(Text):
	color = 15

class EscapedText(Text):
	"""
	An extension to the text class. Special characters can be replaced
	by customized Text objects via the escapeChar method.
	"""
	def __init__(self, *data):
		newdata = []
		for text in data:
			if isinstance(text, Text):
				newdata.append(text)
			else:
				for c in text:
					c = self.escapeChar(c)
					newdata.append(c)
		Text.__init__(self, *newdata)

	def escapeChar(self, char):
		"""
		return a replacement Text object for the character char
		or char itself, if the character should be used as is.
		This method should be overwritten by subclasses.
		"""
		return char
